/***************************************************************************
 *                                                                         *
 *   alignment.h (begin: Feb 20 2003)                                      *
 *                                                                         *
 *   Parallel IQPNNI - Important Quartet Puzzle with NNI                   *
 *                                                                         *
 *   Copyright (C) 2005 by Le Sy Vinh, Bui Quang Minh, Arndt von Haeseler  *
 *   Copyright (C) 2003-2004 by Le Sy Vinh, Arndt von Haeseler             *
 *   {vinh,minh}@cs.uni-duesseldorf.de                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ALI_H
#define ALI_H

#include <iostream>
#include <fstream>

#include "vec.h"
#include "seq.h"
#include "ptn.h"
#include "mat.h"
#include "dvec20.h"
#include "dmat20.h"

using namespace std;

// create the temporary char-BASE map
void createPosGenChar ();


/**
* The Alignment class contains all information and mothods for DNA alignment and Protein alignment,
* Note: The Alignment class does not inherit from Sequence class
* ...Being specified more
*/
class Ali {
public :
	char fileName_[FILENAME_LEN];

	/**
		construction function to init data
	*/
	Ali ();

	/**
		use this function for constructing this class
	*/
	void doConstructor ();

	/**
	 clean all content of this class
	*/
	void clean ();

	/**
	 everything is inited here
	*/
	void init ();

	/**
		detect the data source
	*/
	PAM_TYPE detectDataSource ();

	/**
		detect the data type
	@return NUCLEOTIDE or AMINO_ACID
	*/
	DATA_TYPE detectDataType ();

	/**
	convert the data input file
	*/
	void convertDataType(DATA_TYPE data_type);

	void convertToCodons();
	
	/**
		return the length of the longest sequence 
	@return the length of the longest sequence 
	*/
	int getLongestSeqNameLen ();

	/**
		return the content at the given site, only active sequences are taking into account
	@return the content at the given site, only active sequences are taking into account
	*/
	void getPtn (const int siteNo, Ptn &ptn);

	/**
	return the number sequences of this alignment
	@return the number sequences of this alignment
	*/
	int getNSeq ();

	/**
	return the number of nucleotides in a sequence
	@return the number of nucleotides in a sequence
	*/
	int getNSite ();

	/**
		return the sequence indexth
	@return the sequence indexth
	*/
	Seq &getSeq (const int index);

	/**
		sort sequences by their name, it serves for deburging
	*/
	void sortSeq ();

	/**
		read input from the file fileName
	*/
	int readInput (const char *fileName);


	double getGenDis (const int seqNo1, const int seqNo2);

	/**
	return the address of genetic distance matrix
	*/
	Mat<double> *getGenDis ();

	/**
		create the distance matrix for any two sequences basing on maximum likelihood method
	*/
	void cmpGenDis (char *aliFileName);


	void setPtn (int siteNo, int ptnNo);
	int getPtn (int siteNo);

	//--------------------------------------------------------------------
	/**
	write all information of this alignment 
	*/
	template <class Tout>
	Tout &write (Tout &out) {
		out << nSeq_ << " " << nSite_ << endl;
		for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
			items_[seqNo_].write (out);
		out << "=========end of this alignment====================================" << endl;
		return out;

	}

	//--------------------------------------------------------------------
	/**
	write all information of this alignment 
	*/
	void write (const char *fileName) {
		std::ofstream out_;
		out_.open (fileName);
		out_ << nSeq_ << " " << nSite_ << endl;
		for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++) {
			out_ <<'T' << seqNo_ <<"   ";
			items_[seqNo_].writeItem (out_) << endl;
		}
		out_.close ();
	}

	/**
	get the name of the sequence at certain ndNo
	*/
	Vec<char> &getName (int ndNo);
	/**
	get the name of the sequence at certain ndNo
	*/
	void getName (int ndNo, Vec<char> &name);

	/**
	get the order of the sequence whose name is input name
	*/
	int getSeqNo (Vec<char> &name);

	/**
	set the memory for this class
	*/
	void setLimit (int nSeq, int nSite);

	/**
	get the base frequency of this alignment\
	*/
	void estimateStateFrq (DVec20 &stateFrqArr);

	/**
		estimate the codon frequency
	*/
	void estimateCodonFrq(DVec20 &stateFrqArr);
	
	
	double estimateProb (BASE baseX, BASE baseY);
	void estimateGenPam (double &tsAG, double &tsCT, double &tvAC, double &tvAT, double &tvCG, double &tvGT);
	double estimateTsTvRatio ();

	void getApp (int stateNo, DVec20 &appArr);

	int checkSameName ();

	/**
	release all memmory for this class
	*/
	void release ();


	/**
	destruction function to release data
	*/
	~Ali ();

	/**************************************************************************
	**************************************************************************
	**************************************************************************
	**************************************************************************/

	/**
	check whether or not the genDismat of this class was cmped
	*/
	int isGenDisMatCmped_;
	
private :
	/**
	the number of sequence of this alignment, we do not use it, instead of using nActSeq
	*/
	int nSeq_;
	int longestSeqNameLen_;


	int ptnArr_[MAX_NUM_SITE];

	/**
	the number of sites of this alignment
	*/
	int nSite_;

	/**
	the alignment information
	*/
	Vec<Seq> items_;

	/**
	disMat_[i][j] is the distance between sequence i and sequence j
	*/
	Mat<double> genDisMat_;

}
; //end class ali

extern Ali alignment;

#endif
